@@ -1,9 +1,8 @@
 //===- bitreader.go - Bindings for bitreader ------------------------------===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 //
@@ -15,6 +14,7 @@ package llvm
 
 /*
 #include "llvm-c/BitReader.h"
+#include "llvm-c/Core.h"
 #include <stdlib.h>
 */
 import "C"
@@ -40,7 +40,7 @@ func ParseBitcodeFile(name string) (Module, error) {
 	defer C.LLVMDisposeMemoryBuffer(buf)
 
 	var m Module
-	if C.LLVMParseBitcode(buf, &m.C, &errmsg) == 0 {
+	if C.LLVMParseBitcode2(buf, &m.C) == 0 {
 		return m, nil
 	}
 
