@@ -6,6 +6,13 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "llvm/ADT/ArrayRef.h"
+#include "llvm/ADT/FunctionExtras.h"
+#include "llvm/ADT/Optional.h"
+#include "llvm/ADT/PointerIntPair.h"
+#include "llvm/ADT/SmallString.h"
+#include "llvm/ADT/SmallVector.h"
+#include "llvm/ADT/StringRef.h"
 #include "llvm/Support/type_traits.h"
 #include "gtest/gtest.h"
 
@@ -92,6 +99,28 @@ TEST(Triviality, Tester) {
   TrivialityTester<B &&, false, true>();
 }
 
+// Test that the following ADT behave as expected wrt. trivially copyable trait
+//
+// NB: It is important that this trait behaves the same for (at least) these
+// types for all supported compilers to prevent ABI issue when llvm is compiled
+// with compiler A and an other project using llvm is compiled with compiler B.
+
+TEST(Triviality, ADT) {
+
+  TrivialityTester<llvm::SmallVector<int>, false, false>();
+  TrivialityTester<llvm::SmallString<8>, false, false>();
+
+  TrivialityTester<std::function<int()>, false, false>();
+#if !defined(__FreeBSD__)
+  TrivialityTester<std::pair<int, bool>, true, true>();
+#endif
+  TrivialityTester<llvm::unique_function<int()>, false, false>();
+  TrivialityTester<llvm::StringRef, true, true>();
+  TrivialityTester<llvm::ArrayRef<int>, true, true>();
+  TrivialityTester<llvm::PointerIntPair<int *, 2>, true, true>();
+  TrivialityTester<llvm::Optional<int>, true, true>();
+}
+
 } // namespace triviality
 
 } // end anonymous namespace
